<?php

class base_rpc_server
{
    public function process($request, $response)
    {
        if($request == null) $request  = new base_prism_request;
        if($response == null) $response = new base_prism_response;

        $params = $request->getParams();
        $oauth = $request->getOauth();
        $appInfo = $request->getAppInfo();

        $logData['apilog_id'] = $request->getRequestID();
        $logData['msg_id'] = $params['msg_id'];
        $logData['worker'] = $params['method'];
        $logData['params'] = [
                'request_ip'=>$request->getRequestID(),
                'request_ip'=>$request->getCallerIP(),
                'request_app_info' => $appInfo,
                'request_oauth_info' => $oauth,
                'api_params' => $params,
            ];
        try
        {
            $this->runtimeStart = microtime(true);
            $this->apilogId = kernel::single('system_apilog')->create('response', $logData);
        }
        catch( Exception $e)
        {
            logger::info('apilog_data : '. var_export($logData,true));
        }

        //过滤systemParams，并返回api的配置
        //对systemParams进行判断，没有进行剥离，在下面的__unsetSystemParams方法中剥离的。
        $api = $this->__getApiInfo($params, $response);

        //实例化api class
        $handler = $api['uses'];
        list($class_name, $action_name) = explode('@', $handler);
        $class = new $class_name;

        //从params里面剥离systemParams
        $format = $params['format'];
        $this->__unsetSystemParams($params);

        //验证方法和参数
        try
        {
            $params = $this->checkParams($class, $action_name, $params, $oauth);
            $result = call_user_func([$class, $action_name], $params, $appInfo);

            try
            {
                $this->runtimeStop= microtime(true);
                $runtime = round(($this->runtimeStop - $this->runtimeStart) , 4);
                kernel::single('system_apilog')->update($this->apilogId, 'success', $result, $runtime);
            }
            catch( Exception $e)
            {
                logger::info('update_apilog_data success : '. var_export($result,true));
            }
            return $response->send($result, $format);
        }
        catch(Exception $e)
        {
            $exceptionClass = get_class($e);
            $errorMessage = $e->getMessage();
            return $this->__sendError($response, $method.".runtimeException", $errorMessage, $format, $exceptionClass);
        }
    }

    private function __sendError($response, $code , $errorMessage, $format, $exception)
    {
        try
        {
            $this->runtimeStop= microtime(true);
            $runtime = round(($this->runtimeStop - $this->runtimeStart) , 4);
            kernel::single('system_apilog')->update($this->apilogId, 'fail', ['message'=>$errorMessage,'exception'=>$exception], $runtime);
        }
        catch( Exception $e)
        {
            logger::info('update_apilog_data fail: '. var_export($errorMessage,true));
        }
        return $response->sendError($code, $errorMessage, $format, $exception);
    }

    /**
     * 处理oauth数据，合并到api参数中
     */
    private function __oauthParamsPre($paramsInfos, $params, $oauth)
    {
        foreach( ['shop_id', 'seller_id', 'user_id'] as $column)
        {
            if( $paramsInfos['params'][$column] && $oauth[$column] )
            {
                $params[$column] = $column == 'shop_id' ?  $oauth[$column] : $oauth['accountid'];
            }
        }

        return $params;
    }

    public function checkParams(&$class, &$action_name, &$params, $oauth)
    {
        if( !method_exists( $class, 'getParams' ) )
        {
            throw new RuntimeException('获取参数列表失败');
        }
        $paramsInfos = $class->getParams();

        $params = $this->__oauthParamsPre($paramsInfos, $params, $oauth);

        apiUtil::paramsValidate($params, $paramsInfos);

        //预处理下params
        //转化下数据的格式，比如对数据结构fields进行转化和extends扩展等
        $params = apiUtil::pretreatment($params, $paramsInfos);

        //判断下方法是否存在
        if( !method_exists( $class, $action_name ) )
        {
            throw new RuntimeException('找不到方法 :' . $action_name);
        }
        return $params;
    }

    private function __getApiInfo(&$params, &$response)
    {
        //提取系统参数
        $method    = $params['method'];
        $timestamp = $params['timestamp'];
        $format    = $params['format'];
        $v         = $params['v'];
        $sign_type = $params['sign_type'];
        $sign      = $params['sign'];
        $exception = 'LogicException';

        if( !base_rpc_validate::isValidate($params) )
        {
            return $this->__sendError($response, 'system.systemParams.signError', app::get('base')->_('签名错误'), $format, $exception);
        }

        if( !( $format == 'json' || $format == 'xml' ) )
        {
            return $this->__sendError($response, 'system.systemParams.formatError', app::get('base')->_('返回格式设定必须是json或者xml'), $format, $exception);
        }

        if( !is_numeric($timestamp) )
        {
            return $this->__sendError($response, 'system.systemParams.timestampFormatError', app::get('base')->_('时间格式错误（包含非数字的字符）'), $format, $exception);
        }

        if( time() - intval($timestamp) > 300 )
        {
            return $this->__sendError($response, 'system.systemParams.timeOut', app::get('base')->_('请求已超时'), $format, $exception);
        }

        $apis = config::get('apis.routes');
        if( !isset($apis[$method]) )
        {
            return $this->__sendError($response, 'system.systemParams.methodNotFound', app::get('base')->_('找不到请求API'), $format, $exception);
        }

        if( !in_array($v, $apis[$method]['version']) )
        {
            return $this->__sendError($response, 'system.systemParams.versionNotMatch', app::get('base')->_('API版本不匹配'), $format, $exception);
        }

        return $apis[$method];
    }

    private function __unsetSystemParams(&$params)
    {
        $systemParamKeys = ['method', 'timestamp', 'format', 'v', 'sign_type', 'sign'];
        foreach($systemParamKeys as $key)
        {
            unset($params[$key]);
        }
    }

}
